#-----------------------------------------------------------------------
# File    : coconad.mak
# Contents: build coconad program (on Windows systems)
# Author  : Christian Borgelt
# History : 2013.02.22 file created from eclat.mak
#           2014.03.05 program estpsp added (pattern spectrum estim.)
#           2016.04.20 completed dependencies on header files
#-----------------------------------------------------------------------
THISDIR  = ..\..\coconad\src
UTILDIR  = ..\..\util\src
MATHDIR  = ..\..\math\src
TRACTDIR = ..\..\tract\src

CC       = cl.exe
DEFS     = /D WIN32 /D NDEBUG /D _CONSOLE /D _CRT_SECURE_NO_WARNINGS
CFLAGS   = /nologo /W3 /O2 /GS- $(DEFS) /c $(ADDFLAGS)
INCS     = /I $(UTILDIR) /I $(MATHDIR) /I $(TRACTDIR)

LD       = link.exe
LDFLAGS  = /nologo /subsystem:console /incremental:no
LIBS     = 

TABHDRS  = $(UTILDIR)\tabread.h     $(UTILDIR)\tabwrite.h
TABOBJS  = $(UTILDIR)\tabread.obj   $(UTILDIR)\tabwrite.obj

HDRS     = $(UTILDIR)\fntypes.h     $(UTILDIR)\arrays.h      \
           $(UTILDIR)\memsys.h      $(UTILDIR)\symtab.h      \
           $(TRACTDIR)\tract.h      $(TRACTDIR)\train.h      \
	   $(TRACTDIR)\patspec.h    $(TRACTDIR)\clomax.h     \
	   $(TRACTDIR)\cm4seqs.h    $(TRACTDIR)\cmfilter.h   \
           $(TRACTDIR)\report.h     coconad.h
OBJS     = $(UTILDIR)\arrays.obj    $(UTILDIR)\memsys.obj    \
           $(UTILDIR)\idmap.obj     $(UTILDIR)\escape.obj    \
           $(UTILDIR)\scform.obj    $(TRACTDIR)\taread.obj   \
           $(TRACTDIR)\trnread.obj  $(TRACTDIR)\patspec.obj  \
           $(TRACTDIR)\clomax.obj   $(TRACTDIR)\cm4seqs.obj  \
           $(TRACTDIR)\cmfilter.obj $(TRACTDIR)\repcm.obj    \
           ccnmain.obj
		   
OVLHDRS  = $(UTILDIR)\fntypes.h     $(UTILDIR)\arrays.h      \
           $(UTILDIR)\memsys.h      $(UTILDIR)\symtab.h      \
           $(UTILDIR)\random.h      $(UTILDIR)\error.h       \
           $(TRACTDIR)\tract.h      $(TRACTDIR)\train.h      \
           $(TRACTDIR)\patspec.h    $(TRACTDIR)\clomax.h     \
           $(TRACTDIR)\cmfilter.h   $(TRACTDIR)\report.h     \
           coconad.h
OVLOBJS  = $(UTILDIR)\arrays.obj    $(UTILDIR)\memsys.obj    \
           $(UTILDIR)\idmap.obj     $(UTILDIR)\escape.obj    \
           $(UTILDIR)\scform.obj    $(UTILDIR)\params.obj    \
           $(TRACTDIR)\tard.obj     $(TRACTDIR)\trnrd.obj    \
           $(TRACTDIR)\pspdbl.obj   $(TRACTDIR)\cmdbl.obj    \
           $(TRACTDIR)\cmfilter.obj $(TRACTDIR)\repcmd.obj   \
           ccn-ovl.obj

PSPHDRS  = $(UTILDIR)\fntypes.h     $(UTILDIR)\arrays.h      \
           $(UTILDIR)\memsys.h      $(UTILDIR)\symtab.h      \
           $(UTILDIR)\random.h      $(UTILDIR)\error.h       \
           $(TRACTDIR)\tract.h      $(TRACTDIR)\train.h      \
           $(TRACTDIR)\patspec.h    $(TRACTDIR)\clomax.h     \
           $(TRACTDIR)\cm4seqs.h    $(TRACTDIR)\cmfilter.h   \
           $(TRACTDIR)\report.h     coconad.h ccnpsp.h
PSPOBJS  = $(UTILDIR)\arrays.obj    $(UTILDIR)\memsys.obj    \
           $(UTILDIR)\idmap.obj     $(UTILDIR)\escape.obj    \
           $(UTILDIR)\scform.obj    $(UTILDIR)\random.obj    \
           $(UTILDIR)\sigint.obj    $(MATHDIR)\gamma.obj     \
           $(TRACTDIR)\taread.obj   $(TRACTDIR)\trnrs.obj    \
           $(TRACTDIR)\pspetr.obj   $(TRACTDIR)\clomax.obj   \
           $(TRACTDIR)\cm4seqs.obj  $(TRACTDIR)\cmfilter.obj \
           $(TRACTDIR)\repcm.obj    coconad.obj pspmain.obj

PSOHDRS  = $(UTILDIR)\fntypes.h     $(UTILDIR)\arrays.h     \
           $(UTILDIR)\memsys.h      $(UTILDIR)\symtab.h     \
           $(UTILDIR)\random.h      $(UTILDIR)\error.h      \
           $(TRACTDIR)\tract.h      $(TRACTDIR)\train.h     \
           $(TRACTDIR)\patspec.h    $(TRACTDIR)\clomax.h    \
           $(TRACTDIR)\cmfilter.h   $(TRACTDIR)\report.h    \
           coconad.h ccnpsp.h
PSOOBJS  = $(UTILDIR)\arrays.obj    $(UTILDIR)\memsys.obj   \
           $(UTILDIR)\idmap.obj     $(UTILDIR)\escape.obj   \
           $(UTILDIR)\scform.obj    $(UTILDIR)\random.obj   \
           $(UTILDIR)\sigint.obj    $(MATHDIR)\gamma.obj    \
           $(TRACTDIR)\taread.obj   $(TRACTDIR)\trnrs.obj   \
           $(TRACTDIR)\pspetr.obj   $(TRACTDIR)\clomax.obj  \
           $(TRACTDIR)\cmfilter.obj $(TRACTDIR)\repcm.obj   \
           overlap.obj psp-ovl.obj $(ADDOBJS)

PRGS     = coconad.exe ccn-ovl.exe ccnpsp.exe psp-ovl.exe

#-----------------------------------------------------------------------
# Build Program
#-----------------------------------------------------------------------
all:         $(PRGS)

coconad.exe: $(TABOBJS) $(OBJS) coconad.mak
	$(LD) $(LDFLAGS) $(TABOBJS) $(OBJS) $(LIBS) /out:$@

ccn-ovl.exe: $(TABOBJS) $(OVLOBJS) coconad.mak
	$(LD) $(LDFLAGS) $(TABOBJS) $(OVLOBJS) $(LIBS) /out:$@

ccnpsp.exe:  $(TABOBJS) $(PSPOBJS) coconad.mak
	$(LD) $(LDFLAGS) $(TABOBJS) $(PSPOBJS) $(LIBS) /out:$@

psp-ovl.exe: $(TABOBJS) $(PSOOBJS) coconad.mak
	$(LD) $(LDFLAGS) $(TABOBJS) $(PSOOBJS) $(LIBS) /out:$@

#-----------------------------------------------------------------------
# Main Programs
#-----------------------------------------------------------------------
ccnmain.obj: $(TABHDRS) $(HDRS) coconad.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_MAIN coconad.c /Fo$@

ccn-ovl.obj: $(TABHDRS) $(OVLHDRS) coconad.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_OVERLAP /D CCN_MAIN coconad.c /Fo$@

pspmain.obj: $(TABHDRS) $(PSPHDRS) ccnpsp.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCNPSP_MAIN ccnpsp.c /Fo$@

psp-ovl.obj: $(TABHDRS) $(PSOHDRS) coconad.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_OVERLAP /D CCNPSP_MAIN ccnpsp.c /Fo$@

#-----------------------------------------------------------------------
# CoCoNAD as a module
#-----------------------------------------------------------------------
coconad.obj: $(HDRS) coconad.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT coconad.c /Fo$@

overlap.obj: $(HDRS) coconad.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_OVERLAP /D CCN_ABORT \
              coconad.c /Fo$@

#-----------------------------------------------------------------------
# Pattern Spectrum Functions
#-----------------------------------------------------------------------
ccnpsp.obj:   $(HDRS) $(UTILDIR)\fntypes.h coconad.h ccnpsp.h \
              coconad.c coconad.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT ccnpsp.c /Fo$@

#-----------------------------------------------------------------------
# External Modules
#-----------------------------------------------------------------------
$(UTILDIR)\arrays.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak arrays.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\memsys.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak memsys.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\idmap.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak idmap.obj     ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\escape.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak escape.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\tabread.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak tabread.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\tabwrite.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak tabwrite.obj  ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\scform.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak scform.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\random.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak random.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\sigint.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak sigint.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(UTILDIR)\params.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak params.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(MATHDIR)\gamma.obj:
	cd $(MATHDIR)
	$(MAKE) /f math.mak gamma.obj     ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\taread.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak taread.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\tard.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak tard.obj     ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\trnread.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak trnread.obj  ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\trnrs.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak trnrs.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\trnrd.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak trnrd.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\patspec.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak patspec.obj  ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\pspdbl.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak pspdbl.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\pspetr.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak pspetr.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\clomax.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak clomax.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\cmdbl.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak cmdbl.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\cm4seqs.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak cm4seqs.obj  ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\cmfilter.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak cmfilter.obj ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\cmfdbl.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak cmfdbl.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\repcm.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak repcm.obj    ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)
$(TRACTDIR)\repcmd.obj:
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak repcmd.obj   ADDFLAGS="$(ADDFLAGS)"
	cd $(THISDIR)

#-----------------------------------------------------------------------
# Install
#-----------------------------------------------------------------------
install:
	-@copy coconad.exe ..\..\..\bin
	-@copy estpsp.exe  ..\..\..\bin

#-----------------------------------------------------------------------
# Clean up
#-----------------------------------------------------------------------
localclean:
	-@erase /Q *~ *.obj *.idb *.pch $(PRGS)

clean:
	$(MAKE) /f coconad.mak localclean
	cd $(TRACTDIR)
	$(MAKE) /f tract.mak localclean
	cd $(MATHDIR)
	$(MAKE) /f math.mak clean
	cd $(UTILDIR)
	$(MAKE) /f util.mak clean
	cd $(THISDIR)
