/*----------------------------------------------------------------------
  File    : cmfilter.h
  Contents: linear filter repository for closed and maximal patterns
  Author  : Christian Borgelt
  History : 2016.10.19 file created
            2016.10.21 function cmf_report() added (all item sets)
----------------------------------------------------------------------*/
#ifndef __CMFILTER__
#define __CMFILTER__
#include "tract.h"
#include "report.h"

#ifndef INFINITY
#include <float.h>
#define INFINITY    (DBL_MAX+DBL_MAX)
#endif                          /* in case C99 is not supported */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct {                /* --- closed/maximal filter --- */
  size_t iscnt;                 /* number of stored item sets */
  size_t blkcnt;                /* number of memory blocks */
  size_t blksize;               /* (standard) size of a memory block */
  void   *head;                 /* head of the memory block list */
  void   *end;                  /* end of the current block */
} CMFILTER;                     /* (closed/maximal filter) */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern CMFILTER* cmf_create  (size_t blksize);
extern void      cmf_delete  (CMFILTER *cmf);

extern int       cmf_add     (CMFILTER *cmf,
                              const ITEM *items, ITEM n, RSUPP supp);
extern size_t    cmf_cnt     (CMFILTER *cmf);
extern int       cmf_report  (CMFILTER *cmf, ISREPORT *report);
extern int       cmf_closed  (CMFILTER *cmf, ISREPORT *report);
extern int       cmf_maximal (CMFILTER *cmf, ISREPORT *report);

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define cmf_cnt(r)           ((r)->iscnt)

#endif
